'''
Created on 2013-04-29

@author: Jackie

Task 2 of the EACL paper: question answering
'''

from utilities.io import read_corenlp_corpus, read_corenlp_doc
from utilities.io import read_list, read_dict
from numpy import mean
import numpy as np


def read_scores(fn):
    '''Read the scores contained in fn.'''
    fh = open(fn, 'r')
    lines = fh.read().split('\n')
    cases = []
    for line in lines:
        if len(line) == 0: continue
        correct, scores = line.split('\t', 1)
        correct = int(correct)
        scores = eval(scores)
        cases.append((scores, correct))
    return cases

def compute_norm_ranks(fname):
    cases = read_scores(fname)
    norm_ranks = []
    for case in cases:
        all_scores, correct = case
        ans_score = all_scores[correct] 
        nr = norm_rank(all_scores, ans_score)
        norm_ranks.append(nr)
    return norm_ranks, mean(norm_ranks)

def norm_rank(all_scores, ans_score):
    all_scores = sorted(all_scores) # ascending : lower is worse
    indices = [float(i) for i in xrange(len(all_scores)) if all_scores[i] == ans_score]
    ans_index = mean(indices)
    nr = float(ans_index) / (len(all_scores) - 1)
    return nr
    
        

################################################################################
# I/O and preprocessing code.
################################################################################

def read_qa_items(items_f, questions_f, genia_d, genia_fdict_f):
    '''
    Read the QAItems based on the input files.
    items_f: contains all the correct answers generated by USP.
    questions_f: the xml file containing all of the questions
    genia_d: the directory containing all the GENIA documents.
    '''
    items = read_list(items_f, vf = lambda x: x.split('\t'))[1:] # remove header
    items = [tuple(map(int, item)) for item in items]
    #items = list(set(items))
    
    # an item is a (q_id, doc_id, line_num)
    questions = read_questions(questions_f) # Note: THIS IS 1-INDEXED!!!
    
    # read the relevant documents in GENIA
    docnums = list(set([item[1] for item in items]))
    docnums.sort()
    #print docnums
    genia_fdict = read_dict(genia_fdict_f, kf = int, vf = str)
    docs = None
    if genia_d is not None:
        docs = dict(read_corenlp_corpus(genia_d, genia_fdict, toload = docnums))
        #print len(docs)
    qa_items = []
    for q_id, doc_id, answer in items:
        q = questions[q_id + 1]
        doc = None
        if docs is not None:
            doc = docs[doc_id]
        qa_item = QAItem(q, doc, answer)
        qa_items.append(qa_item)
    return qa_items
     
    

class QAItem:
    def __init__(self, question, doc, answer):
        self.question = question
        self.doc = doc
        self.answer = answer


def read_questions(questions_f):
    '''
    Returns the questions as a dict:
    question number -> Sentence
    '''
    questions = read_corenlp_doc(questions_f, -1)
    return questions.sents

